/*
 AnaGram, a System for Syntax Directed Parsing

 Character Sink Class Definitions
 Copyright (c) 1993, Parsifal Software.
 All Rights Reserved.

*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include "assert.h"
#include "charsink.h"



// Output File Class

character_sink &output_file::printf(char *fmt, ...) {
  va_list argptr;

  assert(file != NULL);
  va_start(argptr, fmt);
  vfprintf(file, fmt, argptr);
  va_end(argptr);
  return *this;
}



// String Accumulator Class


// Constructor

string_accumulator::string_accumulator(int nc, int nx) {
  cs = new char[nc];                 // allocate character storage
  xs = new unsigned[nx];             // allocate index storage
  csx = 0;
  xsx = nx;
  csmax = nc - 1;                    // leave room for a terminating null
  xsmax = nx;
  xs[--xsx] = csx;
  copy_flag = 0;                     // this is live
}


// Copy constructor

string_accumulator::string_accumulator(string_accumulator &s) {
  *this = s;
  copy_flag++;                       // this is memorex
}


// Destructor

string_accumulator::~string_accumulator() {
  if (copy_flag) return;
  delete[] cs;
  delete[] xs;
}


// Reset string accumulator

string_accumulator &reset(string_accumulator &s) {
  assert(s.copy_flag == 0);
  s.csx = 0;
  s.xsx = s.xsmax;
  s.xs[--s.xsx] = s.csx;
  return s;
}


// Append data to stack

character_sink &string_accumulator::operator << (char *str) {
  int k = strlen(str);
  assert(copy_flag == 0);
  assert(csx + k < csmax);           // room for string
  strcpy(&cs[csx],str);
  csx += k;
  return *this;
}

character_sink &string_accumulator::operator << (unsigned char *str) {
  int k = strlen((char *) str);
  assert(copy_flag == 0);
  assert(csx + k < csmax);
  strcpy(&cs[csx],(char *) str);
  csx += k;
  return *this;
}

character_sink &string_accumulator::printf(char *fmt, ...) {
  va_list argptr;
  int n;

  assert(copy_flag == 0);
  va_start(argptr, fmt);
  n = vsprintf(&cs[csx], fmt, argptr);
  va_end(argptr);
  assert(csx + n < csmax);
  csx += n;
  return *this;
}

// Pop character from stack

string_accumulator &string_accumulator::operator >> (int &c) {
  assert(copy_flag == 0);
  assert(csx > xs[xsx]);             // not empty
  c = cs[--csx];                     // pop character
  return *this;
}


// Access character on stack

char &string_accumulator::operator [](unsigned i) {
  assert(i < csx - xs[xsx]);         // index in bounds
  return cs[csx - i - 1];            // identify character
}


// Concatenate strings

string_accumulator &concat(string_accumulator &s) {
  assert(s.copy_flag == 0);
  assert(s.xsx < s.xsmax - 1);       // at least two strings
  s.xsx++;
  return s;
}


// Make permanent copy of string

char *copy(string_accumulator &s) {
  char *c = new char[strlen(s)+1];

  strcpy(c,s);
  return c;
}
