/*
 AnaGram, a System for Syntax Directed Programming
 C Macro preprocessor

 Copyright (c) 1993, Parsifal Software.
 All Rights Reserved.

 Global Definitions
*/

#ifndef MPP_H
#define MPP_H


//#define NDEBUG
#include "token.h"
#include <charsink.h>
#include <stack.h>
#include <strdict.h>
#include "ex.h"

#include "cc.h"

#define N_MACROS 500            // max number of macros
#define N_SYMBOLS 5000          // size of token dictionary

enum symbol_type_enum {identifier, typedef_name};

struct macro_descriptor {
  unsigned name;                // token dictionary index of name
  unsigned* arg_names;          // pointer to list of parameter names
  int n_args;                   // number of parameters
  token *body;                  // body of macro
  unsigned busy_flag : 1;       // set when macro should not be expanded
  unsigned parens : 1;          // set if macro defined with parens
};

struct op_descriptor {
  char *op;
  token_id id;
};


// Parser classes

// Expression evaluator
//  member functions are in EX.SYN

class expression_evaluator : public token_sink {
  ex_pcb_type pcb;
public:
  expression_evaluator();
  friend token_sink& reset(expression_evaluator &);
  token_sink& operator << (token t);
  token_sink& operator << (token *tp);
  operator long();
//  friend int error(expression_evaluator &);
};


// C parser
// Member Functions are in JRC.SYN or KRC.SYN

class c_parser: public token_sink {
public:
  cc_pcb_type pcb;
  c_parser();
 ~c_parser();
  token_sink &operator << (token);
  token_sink& operator << (token *);
//  friend int error(c_parser &p);
  friend c_parser &reset(c_parser &);
};


// Global variables
//  Variables are defined in mpp.cpp

extern  expression_evaluator   condition;
extern  char                   default_path[];
extern  unsigned               defined_value;
extern  int                    if_clause;
extern  macro_descriptor       macro[];
extern  unsigned               macro_id[];
extern  int                    n_macros;
extern  unsigned               n_reserved_words;
extern  int                    nest_comments;
extern  unsigned               one_value;
extern  stack<char *>          paths;
extern  op_descriptor          reserved_words[];
extern  string_accumulator     sa;
extern  token_accumulator      ta;
extern  token_sink            *scanner_sink;
extern  string_dictionary      td;
extern  unsigned               token_handles[];
extern  unsigned               zero_value;


// Function declarations

token_id classify_token(char *);                           // ct.syn
void     expand_macro(token, unsigned);                    // asc.syn
void     expand_text(token *, int = 0, unsigned * = NULL); // asc.syn
void     scan_input(char *);                               // ts.syn
void     syntax_error(char *);                             // ts.syn
int GetLine(void);											//ts.syn

#endif
