/*************************************************************************
**                                                                      **
** These are the node processor routines                                **
**                                                                      **
** Copyright (c) 1994 by Jim Patchell                                   **
**                                                                      **
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <symtab.h>
#include <nodeman.h>
#include <value.h>

/* Misc Global variables */
static int	do_binary_const(value **v1p,int op, value **v2p );

extern value *(*node_proc[])(NODE *n);
static int ConstantExpression = 0;
/*
** node processor routines
*/
enum ass_ops {ASSOP_EQUALS,ASSOP_ADD,ASSOP_SUB,ASSOP_MUL,ASSOP_DIV,
			ASSOP_MOD,ASSOP_OR,ASSOP_AND,ASSOP_XOR,ASSOP_SHL,ASSOP_SHR};

static char *assop_strings[] = {
	"equals",
	"add",
	"sub",
	"mul",
	"div",
	"mod",
	"or",
	"and",
	"xor",
	"shl",
	"shr",
	NULL

};


enum bin_ops {BINOP_ADD,BINOP_SUB,BINOP_MUL,BINOP_DIV,BINOP_MOD,
			BINOP_OR,BINOP_AND,BINOP_XOR,BINOP_SHL,BINOP_SHR};

static char *binop_strings[] = {
	"add",
	"sub",
	"mul",
	"div",
	"mod",
	"or",
	"and",
	"xor",
	"shl",
	"shr",
	NULL
};

static int binop_ops[] = {'+','-','*','/','%','|','&','^','<','>'};

/*-----------------------------------------------------------------------
**
** Assignment operators
**
**---------------------------------------------------------------------*/


static value *assop(NODE *n,int op)
{
	NODE *n1,*n2;		/*	child nodes	*/
	value *v1,*v2;      /*	values returned from processor nodes	*/

printf("ASSIGNOP = %s\n",assop_strings[op]);
	n1 = n->down;
	n2 = n1->next;
	v1 = (node_proc[n1->id])(n1);		/*	process nodes	*/
	v2 = (node_proc[n2->id])(n2);
	return NULL;
}

static value *equals(NODE *n)
{
	return assop(n,ASSOP_EQUALS);
}

static value *muleq(NODE *n)
{
	return assop(n,ASSOP_MUL);
}

static value *diveq(NODE *n)
{
	return assop(n,ASSOP_DIV);
}

static value *modeq(NODE *n)
{
	return assop(n,ASSOP_MOD);
}

static value *addeq(NODE *n)
{
	return assop(n,ASSOP_ADD);
}

static value *subeq(NODE *n)
{
	return assop(n,ASSOP_SUB);
}

static value *shleq(NODE *n)
{
	return assop(n,ASSOP_SHL);
}

static value *shreq(NODE *n)
{
	return assop(n,ASSOP_SHR);
}

static value *andeq(NODE *n)
{
	return assop(n,ASSOP_AND);
}

static value *oreq(NODE *n)
{
	return assop(n,ASSOP_OR);
}

static value *xoreq(NODE *n)
{
	return assop(n,ASSOP_XOR);
}

/*-------------------------------------------------------------------------
**
** Binary operators
**
**------------------------------------------------------------------------*/

static value *binary(NODE *n,int op)
{
	NODE *n1,*n2;		/*	child nodes	*/
	value *v1,*v2;      /*	values returned from processor nodes	*/

printf("BINOP=%s\n",binop_strings[op]);
	n1 = n->down;
	n2 = n1->next;
	v1 = (node_proc[n1->id])(n1);		/*	process nodes	*/
	v2 = (node_proc[n2->id])(n2);
	if(do_binary_const(&v1,binop_ops[op],&v2) )		/*	was this a const	*/
	{
		release_value(v2);
		printf("VALUE=%d\n",v1->type->V_INT);
		return(v1);
	}
	return NULL;
}

static value *question(NODE *n)
{
	/* This one is actually a trinary operator, but who is counting	*/

	return NULL;
}

static value *bitor(NODE *n)
{
	return binary(n,BINOP_OR);
}

static value *bitxor(NODE *n)
{
	return binary(n,BINOP_XOR);
}

static value *bitand(NODE *n)
{
	return binary(n,BINOP_AND);
}

static value *shl(NODE *n)
{
	return binary(n,BINOP_SHL);
}

static value *shr(NODE *n)
{
	return binary(n,BINOP_SHR);
}

static value *add(NODE *n)
{
	return binary(n,BINOP_ADD);;
}

static value *sub(NODE *n)
{
	return binary(n,BINOP_SUB);
}

static value *mul(NODE *n)
{
	return binary(n,BINOP_MUL);
}

static value *div_exp(NODE *n)
{
	return binary(n,BINOP_DIV);
}

static value *mod(NODE *n)
{
	return binary(n,BINOP_MOD);
}

/*-------------------------------------------------------------------------
**
** Logical operators
**
**-----------------------------------------------------------------------*/

static value *or(NODE *n)
{
	return NULL;
}

static value *and(NODE *n)
{
	return NULL;
}

static value *eq(NODE *n)
{
	return NULL;
}

static value *ne(NODE *n)
{
	return NULL;
}

static value *lt(NODE *n)
{
	return NULL;
}

static value *gt(NODE *n)
{
	return NULL;
}

static value *le(NODE *n)
{
	return NULL;
}

static value *ge(NODE *n)
{
	return NULL;
}

/*-------------------------------------------------------------------------
**
** Urnary operators
**
**------------------------------------------------------------------------*/

static value *cast(NODE *n)
{
	return NULL;
}

static value *preinc(NODE *n)
{
	return NULL;
}

static value *predec(NODE *n)
{
	return NULL;
}

static value *complement(NODE *n)
{
	return NULL;
}

static value *address_of(NODE *n)
{
	return NULL;
}

static value *content_of(NODE *n)
{
	return NULL;
}

static value *positive(NODE *n)
{
	NODE *n1;		/*	child nodes	*/
	value *v1;		/*	child value	*/
	link *l;

	n1 = n->down;
	v1 = (node_proc[n1->id])(n1);	/*	process nodes	*/
	l = v1->type;
	return v1;
}

static value *negative(NODE *n)
{
	NODE *n1;		/*	child nodes	*/
	value *v1;		/*	child value	*/
	link *l;

	n1 = n->down;
	v1 = (node_proc[n1->id])(n1);	/*	process nodes	*/
	l = v1->type;
	if(IS_CONSTANT(l))
	{
		if(IS_INT(l))
			l->V_INT = -l->V_INT;
		else if (IS_LONG(l))
			l->V_LONG = -l->V_LONG;
	}
	return v1;
}

static value *not(NODE *n)
{
	return NULL;
}

static value *size_of(NODE *n)
{
	return NULL;
}

static value *array(NODE *n)
{
	return NULL;
}

static value *member(NODE *n)
{
	return NULL;
}

static value *funccall(NODE *n)
{
	return NULL;
}

static value *ptr_to(NODE *n)
{
	return NULL;
}

static value *postinc(NODE *n)
{
	return NULL;
}

static value *postdec(NODE *n)
{
	return NULL;
}

static value *subscript(NODE *n)
{
	return NULL;
}

/*----------------------------------------------------------------------
**
** basic elements that make up the code
**
**--------------------------------------------------------------------*/

static value *constant(NODE *n)
{
	value *v;

	v = make_icon(n->symb,0);
	return v;
}

static value *identifier(NODE *n)
{
	return NULL;
}

static value *string(NODE *n)
{
	return NULL;
}

static value *label(NODE *n)
{
	return NULL;
}

/*----------------------------------------------------------------------
**
** Miscilaneous Node functions
**
**--------------------------------------------------------------------*/

static value *arguments(NODE *n)
{
	return NULL;
}

static value *comma(NODE *n)
{
	return NULL;
}

/*-----------------------------------------------------------------------
**
** Switch Statement node functions
**
**---------------------------------------------------------------------*/

static value *switch_statement(NODE *n)
{
	return NULL;
}

static value *case_statement(NODE *n)
{
	return NULL;
}

static value *default_statement(NODE *n)
{
	return NULL;
}

/*----------------------------------------------------------------------
**
** looping node functions
**
**--------------------------------------------------------------------*/

static value *for_statement(NODE *n)
{
	return NULL;
}

static value *do_statement(NODE *n)
{
	return NULL;
}

static value *while_statement(NODE *n)
{
	return NULL;
}

/*----------------------------------------------------------------------
**
** If statement functions
**
**--------------------------------------------------------------------*/

static value *if_statement(NODE *n)
{
	return NULL;
}

static value *ifelse_statement(NODE *n)
{
	return NULL;
}

static value *ifpart(NODE *n)
{
	return NULL;
}

static value *elsepart(NODE *n)
{
	return NULL;
}

/*---------------------------------------------------------------------
**
** Goto statements
**
**-------------------------------------------------------------------*/

static value *return_statement(NODE *n)
{
	return NULL;
}

static value *break_statement(NODE *n)
{
	return NULL;
}

static value *continue_statement(NODE *n)
{
	return NULL;
}

static value *goto_statement(NODE *n)
{
	return NULL;
}

/*----------------------------------------------------------------------
**
** Expression node functions
**
**--------------------------------------------------------------------*/

static value *expression(NODE *n)
{
	NODE *n1;		/*	child nodes	*/
	value *v1;      /*	values returned from processor nodes	*/

	n1 = n->down;
	while(n1 != NULL)	/*	process the nodes in the tree	*/
	{
		v1 = (node_proc[n1->id])(n1);		/*	process nodes	*/
		n1 = n1->next;
	}
	return v1;
}

static value *null_statement(NODE *n)
{
	return NULL;
}

static value *constant_exp(NODE *n)
{
	NODE *n1;		/*	child nodes	*/
	value *v1;      /*	values returned from processor nodes	*/

	ConstantExpression = 1;	/*	process all following nodes as if constant	*/
	n1 = n->down;
	while(n1 != NULL)	/*	process the nodes in the tree	*/
	{
		v1 = (node_proc[n1->id])(n1);		/*	process nodes	*/
		n1 = n1->next;
	}
	ConstantExpression = 0;	/*	resume normal processing mode				*/
	return v1;
}



/*
** this is the table of node processor routines
*/

value *(*node_proc[])(NODE *n) = {
	equals,
	muleq,
	diveq,
	modeq,
	addeq,
	subeq,
	shleq,
	shreq,
	andeq,
	oreq,
	xoreq,
	question,
	or,
	and,
	bitor,
	bitxor,
	bitand,
	eq,
	ne,
	lt,
	gt,
	le,
	ge,
	shl,
	shr,
	add,
	sub,
	mul,
	div_exp,
	mod,
	cast,
	preinc,
	predec,
	complement,
	address_of,
	content_of,
	positive,
	negative,
	not,
	size_of,
	array,
	member,
	funccall,
	ptr_to,
	postinc,
	postdec,
	subscript,
	arguments,
	constant,
	identifier,
	string,
	comma,
	label,
	case_statement,
	default_statement,
	for_statement,
	do_statement,
	while_statement,
	switch_statement,
	if_statement,
	ifelse_statement,
	return_statement,
	break_statement,
	continue_statement,
	goto_statement,
	expression,
	null_statement,
	constant_exp,
	ifpart,
	elsepart
};


/*-----------------------------------------------------------------------**
**                                                                       **
** process_tree                                                          **
**                                                                       **
** this function is used to go through all the branches in a tree to     **
** process the parse tree that was generated.                            **
**                                                                       **
** returns a value pointer.                                              **
** Takes a pointer to a head node to a parse tree.                       **
**-----------------------------------------------------------------------*/

value *process_tree(NODE *n)
{
	value *v=NULL;

	while(n != NULL)
	{
		printf("Processing Nodes %d\n",n->id);
		v = (node_proc[n->id])(n);
		n = n->next;
	}
	return(v);
}


int EvalConstExp(NODE *n)
{
	value *v;
	int rval;

	ConstantExpression = 1;
	while(n != NULL)
	{
		v = (node_proc[n->id])(n);
		n = n->next;
	}
	ConstantExpression = 0;
	rval = v->type->V_INT;
	release_value(v);
	return rval;
}

/*----------------------------------------------------------------------*/
/*do_binary_const() is -> (C) 1990 Allen I. Holub                       */

static int	do_binary_const(value **v1p,int op, value **v2p )
{
    /* If both operands are constants, do the arithmetic. On exit, *v1p
     * is modified to point at the longer of the two incoming types
     * and the result will be in the last link of *v1p's type chain.
     */

    long  x;
    link  *t1 = (*v1p)->type ;
    link  *t2 = (*v2p)->type ;
    value *tmp;

    /* Note that this code assumes that all fields in the union start at the
     * same address.
     */

    if( IS_CONSTANT(t1) && IS_CONSTANT(t2) )
    {
	if( IS_INT(t1) && IS_INT(t2) )
	{
	    switch( op )
	    {
	    case '+':	t1->V_INT +=  t2->V_INT;	break;
	    case '-':	t1->V_INT -=  t2->V_INT;	break;
	    case '*':	t1->V_INT *=  t2->V_INT;	break;
	    case '&':	t1->V_INT &=  t2->V_INT;	break;
	    case '|':	t1->V_INT |=  t2->V_INT;	break;
	    case '^':	t1->V_INT ^=  t2->V_INT;	break;
	    case '/':	t1->V_INT /=  t2->V_INT;	break;
	    case '%':	t1->V_INT %=  t2->V_INT;	break;
	    case '<':	t1->V_INT <<= t2->V_INT;	break;

	    case '>':	if( IS_UNSIGNED(t1) ) t1->V_UINT >>= t2->V_INT;

			else		      t1->V_INT  >>= t2->V_INT;

			break;
	    }
	    return 1;
	}
	else if( IS_LONG(t1) && IS_LONG(t2) )
	{
	    switch( op )
	    {
	    case '+':	t1->V_LONG +=  t2->V_LONG;	break;
	    case '-':	t1->V_LONG -=  t2->V_LONG;	break;
	    case '*':	t1->V_LONG *=  t2->V_LONG;	break;
	    case '&':	t1->V_LONG &=  t2->V_LONG;	break;
	    case '|':	t1->V_LONG |=  t2->V_LONG;	break;
	    case '^':	t1->V_LONG ^=  t2->V_LONG;	break;
	    case '/':	t1->V_LONG /=  t2->V_LONG;	break;
	    case '%':	t1->V_LONG %=  t2->V_LONG;	break;
	    case '<':	t1->V_LONG <<= t2->V_LONG;	break;

	    case '>':	if( IS_UNSIGNED(t1) ) t1->V_ULONG >>= t2->V_LONG;
			else		      t1->V_LONG  >>= t2->V_LONG;
			break;
	    }
	    return 1;
	}
	else if( IS_LONG(t1) && IS_INT(t2) )
	{
	    switch( op )
	    {
	    case '+':	t1->V_LONG +=  t2->V_INT;	break;
	    case '-':	t1->V_LONG -=  t2->V_INT;	break;
	    case '*':	t1->V_LONG *=  t2->V_INT;	break;
	    case '&':	t1->V_LONG &=  t2->V_INT;	break;
	    case '|':	t1->V_LONG |=  t2->V_INT;	break;
	    case '^':	t1->V_LONG ^=  t2->V_INT;	break;
	    case '/':	t1->V_LONG /=  t2->V_INT;	break;
	    case '%':	t1->V_LONG %=  t2->V_INT;	break;
	    case '<':	t1->V_LONG <<= t2->V_INT;	break;

	    case '>':	if( IS_UNSIGNED(t1) ) t1->V_ULONG >>= t2->V_INT;
			else		      t1->V_LONG  >>= t2->V_INT;
			break;
	    }
	    return 1;
	}
	else if( IS_INT(t1) && IS_LONG(t2) )
	{
	    /* Avoid commutativity problems by doing the arithmetic first,
	     * then swapping the operand values.
	     */

	    switch( op )
	    {

	    case '+':	x = t1->V_INT +  t2->V_LONG;

	    case '-':	x = t1->V_INT -  t2->V_LONG;

	    case '*':	x = t1->V_INT *  t2->V_LONG;

	    case '&':	x = t1->V_INT &  t2->V_LONG;

	    case '|':	x = t1->V_INT |  t2->V_LONG;

	    case '^':	x = t1->V_INT ^  t2->V_LONG;

	    case '/':	x = t1->V_INT /  t2->V_LONG;

	    case '%':	x = t1->V_INT %  t2->V_LONG;

	    case '<':	x = t1->V_INT << t2->V_LONG;

	    case '>':	if( IS_UINT(t1) ) x = t1->V_UINT >> t2->V_LONG;

			else              x = t1->V_INT  >> t2->V_LONG;
			break;
	    }

	    t2->V_LONG = x;	/* Modify v1 to point at the larger   */
	    tmp  = *v1p ;	/* operand by swapping *v1p and *v2p. */
	    *v1p = *v2p ;
	    *v2p = tmp  ;
	    return 1;
	}
    }
    return 0;
}
