/*
 AnaGram, a System for Syntax Directed Programming

 Stack Class Definition

 Copyright (c) 1993, Parsifal Software
 All Rights Reserved

 Please note that the entire class is defined in this module. There is no
 supplementary code module.
*/

#ifndef STACK_H
#define STACK_H

#include "assert.h"

template <class T>

class stack {
protected:
  T *cs;                               // data storage
  unsigned *ls;                        // index (level) storage
  unsigned sx;                         // stack index
  unsigned lx;                         // level index
  unsigned sxmax;                      // max stack index
  unsigned lxmax;                      // max level index
  int copy_flag;                       // is it live or memorex?
public:


// Constructor

  stack(unsigned n, unsigned m = 1) {  // (storage request, levels)
    cs = new T[n];                     // allocate data storage
    ls = new unsigned[m];              // allocate index storage
    lx = lxmax = m;                    // set up for pre-decrement push
    sx = sxmax = n;                    // post-increment pop
    ls[--lx] = sx;                     // init first level
    copy_flag = 0;                     // live
  }

  stack(stack &s) {
    *this = s;
    copy_flag++;                       // memorex
  }


// Destructor

  ~stack() {
    if (copy_flag) return;
    delete [] cs;
    delete [] ls;
  }


// Reset function

  friend stack &reset(stack &s);


// Change levels

  stack &operator ++() {               // next level
    assert(copy_flag == 0);
    assert(lx);                        // not going to overflow
    ls[--lx] = sx;                     // push storage index
    return *this;
  }

  stack operator ++(int x) {
    stack<T> cpy(*this);                  // make copy for return value
    assert(copy_flag == 0);
    assert(lx);
    ls[--lx] = sx;
    return cpy;
  }

  stack &operator --() {               // previous level
    assert(copy_flag == 0);
    sx = ls[lx++];                     // restore storage index
    assert(lx < lxmax);                // no underflow
    return *this;
  }

  stack operator --(int x) {          // previous level
    stack <T> cpy(*this);
    assert(copy_flag == 0);
    sx = ls[lx++];                     // restore storage index
    assert(lx < lxmax);                // no underflow
    return cpy;
  }


// Push data

  stack &operator << (T t) {           // push data
    assert(copy_flag == 0);
    assert(sx && lx < lxmax);          // everything's ok
    cs[--sx] = t;                      // save it
    return *this;
  }


// Pop data

  stack &operator >> (T &t) {          // pop data
    assert(copy_flag == 0);
    assert(lx < lxmax && sx < ls[lx]); // all's well
    t = cs[sx++];                      // get it
    return *this;
  }


// Access stack element

  T &operator[] (unsigned k) {         // access stack element
    assert(lx < lxmax && sx + k < ls[lx]);  // legitimate index
    return cs[sx+k];                   // reference data
  }

  operator T * () {
    return &cs[sx];                    // pointer to top element
  }


// Get size of stack

  friend unsigned size(stack <T> &s);      // size at current level

};

template <class T>
inline stack <T> &reset(stack <T> &s) {
  assert(s.copy_flag == 0);
  s.lx = s.lxmax;                    // set up for pre-decrement push
  s.sx = s.sxmax;                    // post-increment pop
  s.ls[--s.lx] = s.sx;               // init first level
  return s;
}

template <class T>
inline unsigned size(stack <T> &s) {     // size at current level
  return s.ls[s.lx] - s.sx;
}

#endif
