/*
 AnaGram, a System for Syntax Directed Programming

 Utility Class Definitions
 For copying, clearing, protecting and duplicating arrays.

 Copyright (c) 1993, Parsifal Software.
 All Rights Reserved.


 These class definitions are complete within this file.
 There is no corresponding CPP module.

*/

#ifndef UTIL_H
#define UTIL_H

#include <string.h>

// General purpose copy procedure. Works with pointers of any type.

template<class T>
void copy(T *dest, T *src, unsigned n = 1) {
  memmove(dest, src, n*sizeof(T));
}

// Clear an array to zero

template <class T>
T *clear(T *dest, unsigned n = 1) {
	memset(dest, 0, n*sizeof(T));
	return dest;
}

// Make a duplicate of any array

template <class T>
T *memdup(T *src, unsigned n) {
  T *result = new T[n];
  memmove(result, src, n*sizeof(T));
  return result;
}

// General purpose class for saving and restoring data arrays

template <class T>
class protect {
	T *save_value;
	unsigned n;
	T *original_pointer;
public:
  protect(T *p, unsigned k = 1) {
    save_value = new T[n = k];
    copy(save_value, p, n);
    original_pointer = p;
  }
  ~protect() {
    copy(original_pointer, save_value, n);
    delete [] save_value;
  }
};

#endif
